from GConfBackend import GConfBackend


_CONFIG_LOCATION = "/apps/gdesklets"


#
# Class for managing configuration.
#
class ConfigManager:

    def __init__(self):

        self.__cache = {}
        self.__backend = GConfBackend(_CONFIG_LOCATION)
        self.__backend.add_observer(self.__on_observe_backend)



    #
    # Cache is updated when the configuration backend calls for an update
    #
    def __on_observe_backend(self, src, cmd, *args):

        if (cmd == src.OBS_UPDATE):
            key, value = args
            config_id, k = key

            if (not self.__cache.has_key(config_id)):
                self.__cache[config_id] = {}
            self.__cache[config_id][k] = value



    #
    # Sets a configuration value
    #
    def set(self, config_id, key, value):

        self.__backend.set(config_id, key, value)



    #
    # Gets a configuration value
    #
    def get(self, config_id, key):

        return self.__backend.get(config_id, key)



    #
    # Sets a callback handler for watching changes for the given configuration
    # entry
    #
    def watch(self, config_id, key, handler):

        self.__backend.watch(config_id, key, handler)



    #
    # Clears an ID entry in the configuration backend
    #
    def clear(self, config_id):

        self.__backend.clear(config_id)



    #
    # Lists directories in the configuration backend
    #
    def list(self, *args):

        return apply(self.__backend.list, args)
