from main import HOME, NAME
from factory.DisplayFactory import DisplayFactory
from config.ConfigManager import ConfigManager
from utils import singleton
from utils import dialog
import admin

from main import _

import os
import gtk


# This class starts, restarts and kills Displays
class Starter:

    def __init__(self):

        # the set of open displays as a hashtable "id -> display"
        self.__open_displays = {}

        self.__factory = singleton.get(DisplayFactory)
        self.__config = singleton.get(ConfigManager)


        # watches main/displays in the configuration base and calls on_watch if
        # something's changed
        self.__config.watch("main", "displays", self.__on_watch)

        # gets current displays from the configuration base
        displays = self.__config.get("main", "displays")

        # check whether there are already some displays; if there aren't any
        # welcome new users
        if (not displays): self.__first_run()



    #
    # Welcomes new users and sets up the configuration base.
    #
    def __first_run(self):

        name = "<i>" + NAME + "</i>"
        dialog.info(_("Welcome to %(name)s<i>!</i>") % vars(),
                    _("It looks like you are running %(name)s for the first "
                      "time.\n"
                      "\n"
                      "You can add new displays by launching them in your "
                      "file manager.\n"
                      "\n"
                      "<b>Have fun,</b>\n"
                      "  <i>the %(name)s developers</i>") % vars())

        self.__config.set("main", "displays", "")



    #
    # Reacts on observer messages from the display.
    #
    def __on_display_action(self, src, cmd, *args):

        if (cmd == src.OBS_CLOSE):
            id = args[0]
            admin.remove_display(id)
            del src

        elif (cmd == src.OBS_RESTART):
            id = args[0]
            admin.restart_display(id)
            del src

        #end if



    #
    # Reacts on changes in the list of displays.
    #
    def __on_watch(self, key, value):

        displays = admin.get_displays()

        # check whether displays have been added
        for id in displays.keys():
            if (not self.__open_displays.has_key(id)):
                self.__add_display(id, displays[id])
        #end for

        # check whether displays have been removed
        for id in self.__open_displays.keys():
            if (not displays.has_key(id)):
                self.__remove_display(id)
        #end for



    #
    # Adds the given display.
    #
    def __add_display(self, id, path):

        if (not self.__open_displays.has_key(id)):
            dsp = self.__create_display(id, path)
            self.__open_displays[id] = dsp

            if (dsp):
                print "adding", id
                self.__open_displays[id] = dsp
                dsp.add_observer(self.__on_display_action)
        #end if



    #
    # Creates and returns a new display from the given data, or None in case
    # of an error.
    #
    def __create_display(self, id, path):

        try:
            fd = open(path, "r")
        except:
            dialog.warning(_("Could not open display file '%(path)s'")
                           % vars(),
                           _("The display file could not be opened because "
                             "the file was not readable."))
            admin.remove_display(id)
            return None

        data = fd.read()
        fd.close()

        oldcwd = os.getcwd()
        dir = os.path.dirname(path)

        os.chdir(dir)
        display = self.__factory.create_display(id, data)

        if (not display):
            dialog.warning(_("Invalid display file '%(path)s'") % vars(),
                           _("The display file contains invalid data and "
                             "could not be loaded."))
            admin.remove_display(id)

        return display



    #
    # Removes the given display.
    #
    def __remove_display(self, id):

        if (self.__open_displays.has_key(id)):
            print "closing", id
            if (self.__open_displays[id]):
                self.__open_displays[id].remove_display()
            del self.__open_displays[id]
        import gc; gc.collect()

        gtk.timeout_add(500, self.__purge_config)



    #
    # Purges unused config entries.
    #
    # TODO: implement it in a more efficient way
    #
    def __purge_config(self):

        for dir in self.__config.list():
            if (not dir[0].isdigit()): continue

            to_remove = 1
            for id in self.__open_displays.keys():
                if (dir.startswith(id)):
                    to_remove = 0
                    break
            #end for

            if (to_remove):
                print "purging", dir
                self.__config.clear(dir)
        #end for



    #
    # Starts up all displays.
    #
    def start_displays(self):

        displays = self.__config.get("main", "displays")
        self.__on_watch(None, displays)
